CoAP Component for Link SDK V4.0.0

# 功能描述
Link-SDK提供了CoAP上云实现, 并和阿里的服务端做了很多性能和稳定性方面的优化, 推荐使用阿里自有的CoAP方案向云平台上报数据

## 限制说明

+ CoAP通信方式适合单纯的数据上报的场景
+ CoAP服务器地址为 `coap.cn-shanghai.link.aliyuncs.com`
+ 目前, 仅中国(上海)地域支持CoAP通信数据上报
+ Topic规范和MQTT的Topic规范一致
    - 上报数据请求: `/topic/${topic}`
    - Topic变量`${topic}`的值可以与MQTT连接通信的Topic相复用
+ 设备认证返回的token会在一定周期后失效(目前token有效期是7天), SDK遇到token失效, 需要用户重新获取

# API列表
以下是完整的CoAP上报部分API列表及简要说明 (详见`coap/aiot_coap_api.h`)

| **接口名**                                                                                                              | **说明**
|-------------------------------------------------------------------------------------------------------------------------|-----------------------------------------------------------------------------------------------------------------------------------------------------
| [aiot_coap_init]()    | 创建一个CoAP上云实例并设置默认参数
| [aiot_coap_setopt]()  | 设置CoAP实例选项, 详见[CoAP设置参数说明]()
| [aiot_coap_auth]()    | 向服务器发送身份认证请求, 获取token
| [aiot_coap_send]()    | 上报数据到物联网平台
| [aiot_coap_recv]()    | 接受CoAP应答数据, 将应答数据保存到用户缓冲区中
| [aiot_coap_deinit]()  | 销毁参数所指定的CoAP实例, 回收资源

# 例程讲解
> 现对照`/coap/demos/coap_basic_demo.c`例程, 分步骤讲解如何使用API

<img src="" width="800">

这个例程适用于`Linux`这类支持pthread的POSIX设备, 它演示了用SDK配置CoAP参数并发起认证, 之后

+ 如果认证失败了, 销毁实例, 回收资源, 结束程序退出
+ 如果成功, 在`demo_coap_post_lightswitch()`不被注释的情况下, 演示上报`1`条消息到云平台

需要用户关注或修改的部分, 已用 `TODO` 在注释中标明

---

## 设置设备的三元组
例程使用的三元组是公用的, 所以应用于实际业务时, 请替换如下的`TODO`部分, 传入用户自己真实的三元组

```c
    /* TODO: 替换为自己设备的三元组 */
    char *product_key = "a13FN5TplKq";
    char *device_name = "coap_basic_demo";
    char *device_secret = "TGijLt2jVngB53pwQVdGvdGXYR5pDH4y";
```

## 进入程序入口, 给SDK配置全局的底层依赖和日志回调
底层依赖描述了硬件平台的资源使用方式, 比如怎样获取时钟, 分配内存等, 日志回调是用户的函数, SDK有log输出的时候会进入这个函数

```c
int main(int argc, char *argv[])
{
    ...
    ...
    /* 配置SDK的底层依赖 */
    aiot_sysdep_set_portfile(&g_aiot_sysdep_portfile);
    /* 配置SDK的日志输出 */
    aiot_state_set_logcb(demo_state_logcb);
```

## 给CoAP上云配置连接参数
这些连接参数包括如何建立TLS连接, 服务器地址在哪, 连接时的三元组是什么, 连接后的数据及事件回调函数是什么等等

```c
    /* 创建1个CoAP客户端实例并内部初始化默认参数 */
    coap_handle = aiot_coap_init();
    /* 配置连接的服务器地址 */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_HOST, host);

    /* 配置连接的服务器端口 */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_PORT, &port);

    /* 配置设备productKey */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_PRODUCT_KEY, (void *)product_key);

    /* 配置设备deviceName */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_DEVICE_NAME, (void *)device_name);

    /* 配置设备deviceSecret */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_DEVICE_SECRET, (void *)device_secret);

    /* 配置网络连接的安全凭据, 上面已经创建好了 */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_CRED, (void *)(&cred));

    /* 配置COAP默认消息接收回调函数 */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_RECV_HANDLER, demo_coap_recv_handler);

    /* 配置COAP事件回调函数 */
    aiot_coap_setopt(coap_handle, AIOT_COAPOPT_EVENT_HANDLER, demo_coap_event_handler);
    ...
    ...
```

## 向服务器发起连接
使用`aiot_coap_auth()`接口, 向CoAP服务器发起身份认证请求

```c
    /* 设备认证, 获取token */
    res = aiot_coap_auth(coap_handle);
    if (res == 0) {
        printf("aiot_coap_auth succeed\r\n");
    } else {
        /* 如果认证失败, 就销毁实例结束程序退出, 实际产品中可根据应用需要进行重试 */
        printf("aiot_coap_auth failed, res = -0x%04x\r\n", -res);
        aiot_coap_deinit(&coap_handle);
        return -1;
    }
```

## 认证成功后, 可尝试上报`1`条业务消息
如果演示函数`demo_coap_post_lightswitch()`处于不被注释掉的状态, 它会上报`1`条业务消息

```c
    /* TODO: 以下代码如果不被注释, 会演示上报 1 次数据到物联网平台 */
    demo_coap_post_lightswitch(coap_handle, product_key, device_name);
```

## 上报这`1`条业务消息的日志
以下文本是由SDK传出, 在`demo_state_logcb()`中打印, 若需关闭可筛选入参`code`为`0x040B`的情况

```
establish mbedtls connection with server(host='a13FN5TplKq.coap.cn-shanghai.link.aliyuncs.com', port=[5684])
success to establish tcp, fd=5
success to establish mbedtls connection, fd = 5(cost 35288 bytes in total, max used 36234 bytes)

[LK-0801] < 7B 22 74 6F 6B 65 6E 22  3A 22 5E 31 5E 31 35 39 | {"token":"^1^159
[LK-0801] < 36 36 31 34 32 30 39 33  36 30 5E 39 30 32 31 32 | 6614209360^90212
[LK-0801] < 34 35 31 62 31 33 39 35  35 66 22 7D             | 451b13955f"}

[1596614211.389][LK-0839] {"token":"^1^1596614209360^90212451b13955f"}
event: auth token received
```

## 云平台接收到这`1`条业务消息后的回应
以下文本是由SDK传出, 在`demo_state_logcb()`中打印, 若需关闭可筛选入参`code`为`0x040B`的情况

```
[LK-0801] > 7B 22 69 64 22 3A 22 31  22 2C 22 76 65 72 73 69 | {"id":"1","versi
[LK-0801] > 6F 6E 22 3A 22 31 2E 30  22 2C 22 70 61 72 61 6D | on":"1.0","param
[LK-0801] > 73 22 3A 7B 22 4C 69 67  68 74 53 77 69 74 63 68 | s":{"LightSwitch
[LK-0801] > 22 3A 30 7D 7D                                   | ":0}}


[LK-0801] < 7B 22 63 6F 64 65 22 3A  32 30 30 2C 22 64 61 74 | {"code":200,"dat
[LK-0801] < 61 22 3A 7B 7D 2C 22 69  64 22 3A 22 31 22 2C 22 | a":{},"id":"1","
[LK-0801] < 6D 65 73 73 61 67 65 22  3A 22 73 75 63 63 65 73 | message":"succes
[LK-0801] < 73 22 2C 22 6D 65 74 68  6F 64 22 3A 22 74 68 69 | s","method":"thi
[LK-0801] < 6E 67 2E 65 76 65 6E 74  2E 70 72 6F 70 65 72 74 | ng.event.propert
[LK-0801] < 79 2E 70 6F 73 74 22 2C  22 76 65 72 73 69 6F 6E | y.post","version
[LK-0801] < 22 3A 22 31 2E 30 22 7D                          | ":"1.0"}

[1596614213.484][LK-083A] {"code":200,"data":{},"id":"1","message":"success","method":"thing.event.property.post","version":"1.0"}
```

## 最后销毁实例, 回收资源, 退出演示程序
CoAP上报不是长连接协议, 保持客户端连接也无法接收服务端的消息, 所以例程无论是否上报, 接下来都会正常结束

```c
    /* 销毁CoAP实例 */
    aiot_coap_deinit(&coap_handle);
    printf("program exit as normal return\r\n");
    printf("\r\n");

    return 0;
}
```

